@@ -571,6 +571,7 @@ extern Fl_Check_Button *poll_noise;
 extern Fl_Check_Button *poll_nr;
 extern Fl_Check_Button *poll_compression;
 extern Fl_Check_Button *poll_ptt;
+extern Fl_Check_Button *poll_tuner;
 extern Fl_Check_Button *poll_break_in;
 extern Fl_Button *btnClearAddControls;
 extern Fl_Check_Button *poll_all;
@@ -640,4 +641,6 @@ extern Fl_ComboBox *cbo_tmate2_dev;
 
 extern Fl_Group *tabs;
 
+extern void exec_btnUser(int val, bool shift = 0);
+
 #endif
