@@ -90,6 +90,7 @@
 #include "yaesu/FT100D.h"
 #include "yaesu/FT450.h"
 #include "yaesu/FT450D.h"
+#include "yaesu/FT710.h"
 #include "yaesu/FT736R.h"
 #include "yaesu/FT747.h"
 #include "yaesu/FT767.h"
@@ -234,5 +235,6 @@ extern RIG_TCI_SUNDX	rig_tci_sundx;	// 103
 extern RIG_TCI_SUNPRO	rig_tci_sunpro;	// 104
 extern RIG_TRUSDX		rig_trusdx;		// 105
 extern RIG_QDX			rig_qdx;		// 106
+extern RIG_FT710		rig_FT710;		// 107
 
 #endif
