@@ -21,8 +21,8 @@
 #ifndef THREADS_H_
 #define THREADS_H_
 
+#include "config.h"
 #include <iostream>
-#include <config.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -98,9 +98,9 @@ bool thread_in_list(int id, const int* list);
 
 
 // On POSIX systems we cancel threads by sending them SIGUSR2,
-// which will also interrupt blocking calls.  On woe32 we use
+// which will also interrupt blocking calls.  On WIN32 we use
 // pthread_cancel and there is no good/sane way to interrupt.
-#ifndef __WOE32__
+#ifndef __WIN32__
 
 #include <signal.h>
 
