@@ -26,19 +26,21 @@
 const char K4name_[] = "K4";
 
 // 0=N/A, 1=LSB, 2=USB, 3=CW, 4=FM, 5=AM, 6=DATA, 7=CW REV, 8=N/A, 9=DATA REV.
-const char *K4modes_[] =
-	{ "LSB", "USB", "CW", "FM", "AM", "DATA", "CW-R", "DATA-R", NULL};
+static std::vector<std::string>K4modes_;
+static const char *vK4modes_[] =
+	{ "LSB", "USB", "CW", "FM", "AM", "DATA", "CW-R", "DATA-R"};
 const char modenbr[] =
 	{ '1', '2', '3', '4', '5', '6', '7', '9' };
 static const char K4_mode_type[] =
 	{ 'L', 'U', 'L', 'U', 'U', 'U', 'U', 'L' };
 
-static const char *K4_widths[] = {
+static std::vector<std::string>K4_widths;
+static const char *vK4_widths[] = {
    "50",  "100",  "150",  "200",  "250",  "300",  "350",  "400",  "450",  "500",
   "550",  "600",  "650",  "700",  "750",  "800",  "850",  "900",  "950", "1000",
  "1100", "1200", "1300", "1400", "1500", "1600", "1700", "1800", "1900", "2000",
  "2200", "2400", "2600", "2800", "3000", "3100", "3200", "3300", "3400", "3500",
- "3600", "3700", "3800", "3900", "4000", NULL};
+ "3600", "3700", "3800", "3900", "4000"};
 static int K4_bw_vals[] = {
  1, 2, 3, 4, 5, 6, 7, 8, 9,10,
 11,12,13,14,15,16,17,18,19,20,
@@ -46,7 +48,9 @@ static int K4_bw_vals[] = {
 31,32,33,34,35,36,37,38, 39,40,
 41,42,43,44,45, WVALS_LIMIT};
 
-static int def_mode_width[] = { 34, 34, 15, 37, 37, 34, 15, 34 };
+static int mode_bwA[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
+static int mode_bwB[] = { -1, -1, -1, -1, -1, -1, -1, -1 };
+static int mode_def_bw[] =  { 2800, 2800, 800, 3600, 3600, 2800, 800, 2800 };
 
 static GUI k4_widgets[]= {
 	{ (Fl_Widget *)btnVol, 2, 125,  50 },
@@ -96,14 +100,15 @@ RIG_K4::RIG_K4() {
 
 	def_freq = freqA = freqB = 14070000ULL;
 	def_mode = modeA = modeB = 1;
-	def_bw = bwA = bwB = 34;
+	def_bw = bwA = bwB = 3000;
 
 	can_change_alt_vfo =
 
 	has_split_AB = true;
 	has_micgain_control =
 	has_rf_control = true;
-	has_bandwidth_control =
+//	has_bandwidth_control =
+	has_int_bandwidth_control =
 	has_power_control = true;
 	has_volume_control =
 	has_mode_control =
@@ -134,19 +139,30 @@ RIG_K4::RIG_K4() {
 
 int  RIG_K4::adjust_bandwidth(int m)
 {
-	return def_mode_width[m];
+	int width = (inuse == onA ? mode_bwA[m] : mode_bwB[m]);
+	if (width == -1)
+		return mode_def_bw[m];
+	return width;
 }
 
 int  RIG_K4::def_bandwidth(int m)
 {
-	return def_mode_width[m];
+	int width = (inuse == onA ? mode_bwA[m] : mode_bwB[m]);
+	if (width == -1)
+		return mode_def_bw[m];
+	return width;
 }
 
 #define K4_WAIT_TIME 800
 
 void RIG_K4::initialize()
 {
-	debug::level = debug::INFO_LEVEL;
+	VECTOR (K4modes_, vK4modes_);
+	VECTOR (K4_widths, vK4_widths);
+
+	modes_ = K4modes_;
+	bandwidths_ = K4_widths;
+	bw_vals_ = K4_bw_vals;
 
 	LOG_INFO("K4");
 	k4_widgets[0].W = btnVol;
@@ -915,111 +931,66 @@ int RIG_K4::get_noise()
 	return (replystr[p+2] == '1' ? 1 : 0);
 }
 
-// FW $ (Filter Bandwidth and Number; GET/SET)
-// K4 Extended SET/RSP format (K41): FWxxxx; where xxxx is 0-9999, the bandwidth
-// in 10-Hz units. May be quantized and/or range limited based on the present
-// operating mode.
+// BW $ (Filter Bandwidth and Number; GET/SET)
+// KX3 Extended SET/RSP format (K31): BWxxxx; where xxxx is 0-9999, the bandwidth in 10-Hz units. May be
+// quantized and/or range limited based on the present operating mode.
+// BW00050 => BW0005;
+// 50 - 1000 in 50 Hz increments
+// 1000 - 3000 in 100 Hz increments
+// 3000 - 4000 in 200 Hz increments
 
 void RIG_K4::set_bwA(int val)
 {
-    char command[10];
-    cmd = "BW";
-    bwA = val;
-    if (val < (int)(sizeof(K4_widths)/sizeof(char*)-2)) // then it's an index
-    {
-      std::string w = K4_widths[val];
-      w.erase(w.length() - 1);
-      while (w.length() < 4) w.insert(0, "0");
-      cmd.append(w).append(";");
-    }
-    else // otherwise it's s real width
-    {
-      short bw = val;
-      if (bw > 4000) bw = 4000;
-      snprintf(command, sizeof(command), "BW%04d;", (bw/10));
-      cmd = command;
-    }
+	char command[10];
+	short bw = val;
+	if (bw > 4000) bw = 4000;
+	snprintf(command, sizeof(command), "BW%04d;", (bw/10));
+	cmd = command;
 
-    set_trace(1, "set bwA");
-    sendCommand(cmd);
-    sett("");
-    set_pbt_values(val);
+	set_trace(1, "set bwA");
+	sendCommand(cmd);
+	sett("");
+	bwA = val;
+	mode_bwA[vfoA.imode] = bwA;
 }
 
 int RIG_K4::get_bwA()
 {
 	cmd = "BW;";
 	get_trace(1, "get bwA val");
-	wait_char(';', 7, K4_WAIT_TIME, "get bwA val", ASC);
+	wait_char(';', 7, KX3_WAIT_TIME, "get bwA val", ASC);
 	gett("");
-
-	size_t p = replystr.rfind("BW");
-	if (p != std::string::npos) {
-		bwA_val = atoi(&replystr[2]) * 10;
-    }
-
-	cmd = "FW;";
-	get_trace(1, "get bwA");
-	wait_char(';', 7, K4_WAIT_TIME, "get bandwidth A", ASC);
-	gett("");
- 
-	p = replystr.rfind("FW");
-	if (p == std::string::npos) return bwA;
-	int bw = atoi(&replystr[3]) * 10;
-	for (bwA = 0; bwA < 36; bwA++)
-		if (bw <= atoi(K4_widths[bwA])) break;
-	return bwA;
+	int bw = bwA / 10;
+	sscanf(replystr.c_str(), "BW%d", &bw);
+	bw *= 10;
+	return bwA = bw;
 }
 
 void RIG_K4::set_bwB(int val)
 {
-    char command[10];
-    cmd = "BW$";
-    bwA = val;
-    if (val < (int)(sizeof(K4_widths)/sizeof(char*)-2)) // then it's an index
-    {
-      std::string w = K4_widths[val];
-      w.erase(w.length() - 1);
-      while (w.length() < 4) w.insert(0, "0");
-      cmd.append(w).append(";");
-    }
-    else // otherwise it's s real width
-    {
-      short bw = val;
-      if (bw > 4000) bw = 4000;
-      snprintf(command, sizeof(command), "BW$%04d;", (bw/10));
-      cmd = command;
-    }
+	char command[10];
+	short bw = val;
+	if (bw > 4000) bw = 4000;
+	snprintf(command, sizeof(command), "BW$%04d;", (bw/10));
+	cmd = command;
 
-    set_trace(1, "set bwB");
-    sendCommand(cmd);
-    sett("");
-    set_pbt_values(val);
+	set_trace(1, "set bwB");
+	sendCommand(cmd);
+	sett("");
+	bwB = val;
+	mode_bwB[vfoB.imode] = bwB;
 }
 
 int RIG_K4::get_bwB()
 {
 	cmd = "BW$;";
 	get_trace(1, "get bwB val");
-	wait_char(';', 8, K4_WAIT_TIME, "get bwB val", ASC);
+	wait_char(';', 7, KX3_WAIT_TIME, "get bwB val", ASC);
 	gett("");
-
-	size_t p = replystr.rfind("BW$");
-	if (p != std::string::npos) {
-		bwB_val = atoi(&replystr[3]) * 10;
-    }
-
-	cmd = "FW$;";
-	get_trace(1, "get bwB");
-	wait_char(';', 8, K4_WAIT_TIME, "get bandwidth B", ASC);
-	gett("");
- 
-	p = replystr.rfind("FW$");
-	if (p == std::string::npos) return bwB;
-	int bw = atoi(&replystr[3]) * 10;
-	for (bwB = 0; bwB < 36; bwB++)
-		if (bw <= atoi(K4_widths[bwB])) break;
-	return bwB;
+	int bw = bwB /10;
+	sscanf(replystr.c_str(), "BW$%d", &bw);
+	bw *= 10;
+	return bwB = bw;
 }
 
 int RIG_K4::get_power_out()
