@@ -50,11 +50,13 @@ static int mode_filterB[NUM_MODES] = {
 	1,1,1,
 	1,1,1};
 
-const char *IC7851modes_[] = {
+static std::vector<std::string>IC7851modes_;
+static const char *vIC7851modes_[] =
+{
 	"LSB", "USB", "AM", "CW", "RTTY",
 	"FM", "CW-R", "RTTY-R", "PSK", "PSK-R", 
 	"LSB-D1", "LSB-D2", "LSB-D3",
-	"USB-D1", "USB-D2", "USB-D3", NULL};
+	"USB-D1", "USB-D2", "USB-D3"};
 
 const char IC7851_mode_type[] = {
 	'L', 'U', 'U', 'U', 'L',
@@ -68,12 +70,14 @@ const char IC7851_mode_nbr[] = {
 	0x00, 0x00, 0x00,
 	0x01, 0x01, 0x01 };
 
-const char *IC7851_ssb_bws[] = {
+static std::vector<std::string>IC7851_ssb_bws;
+static const char *vIC7851_ssb_bws[] =
+{
 "50",    "100",  "150",  "200",  "250",  "300",  "350",  "400",  "450",  "500",
 "600",   "700",  "800",  "900", "1000", "1100", "1200", "1300", "1400", "1500",
 "1600", "1700", "1800", "1900", "2000", "2100", "2200", "2300", "2400", "2500",
 "2600", "2700", "2800", "2900", "3000", "3100", "3200", "3300", "3400", "3500",
-"3600", NULL };
+"3600" };
 
 static int IC7851_bw_vals_SSB[] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
@@ -82,12 +86,14 @@ static int IC7851_bw_vals_SSB[] = {
 30,31,32,33,34,35,36,37,38,39,
 40, WVALS_LIMIT};
 
-const char *IC7851_am_bws[] = {
+static std::vector<std::string>IC7851_am_bws;
+static const char *vIC7851_am_bws[] =
+{
 "200",   "400",  "600",  "800", "1000", "1200", "1400", "1600", "1800", "2000",
 "2200", "2400", "2600", "2800", "3000", "3200", "3400", "3600", "3800", "4000",
 "4200", "4400", "4600", "4800", "5000", "5200", "5400", "5600", "5800", "6000",
 "6200", "6400", "6600", "6800", "7000", "_7851", "7400", "_7851", "_7851", "8000",
-"8200", "8400", "8600", "8800", "9000", "9200", "9400", "9600", "9800", "10000", NULL };
+"8200", "8400", "8600", "8800", "9000", "9200", "9400", "9600", "9800", "10000" };
 
 static int IC7851_bw_vals_AM[] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
@@ -97,7 +103,9 @@ static int IC7851_bw_vals_AM[] = {
 40,41,42,43,44,45,46,47,48,49,
 WVALS_LIMIT};
 
-const char *IC7851_fm_bws[] = { "FIXED", NULL };
+static std::vector<std::string>IC7851_fm_bws;
+static const char *vIC7851_fm_bws[] =
+{ "FIXED" };
 static int IC7851_bw_vals_FM[] = { 1, WVALS_LIMIT};
 
 static GUI IC7851_widgets[]= {
@@ -121,6 +129,17 @@ static GUI IC7851_widgets[]= {
 
 void RIG_IC7851::initialize()
 {
+	VECTOR (IC7851modes_, vIC7851modes_);
+	VECTOR (IC7851_ssb_bws, vIC7851_ssb_bws);
+	VECTOR (IC7851_am_bws, vIC7851_am_bws);
+	VECTOR (IC7851_fm_bws, vIC7851_fm_bws);
+
+	modes_ = IC7851modes_;
+	bandwidths_ = IC7851_ssb_bws;
+	bw_vals_ = IC7851_bw_vals_SSB;
+
+	_mode_type = IC7851_mode_type;
+
 	IC7851_widgets[0].W = btnVol;
 	IC7851_widgets[1].W = sldrVOLUME;
 	IC7851_widgets[2].W = btnAGC;
@@ -457,7 +476,7 @@ void RIG_IC7851::set_modeA(int val)
 
 	set_trace(4, 
 		"set mode A[",
-		IC7851modes_[A.imode], 
+		IC7851modes_[A.imode].c_str(), 
 		"] ", 
 		str2hex(cmd.c_str(), cmd.length()));
 }
@@ -523,7 +542,7 @@ int RIG_IC7851::get_modeA()
 end_wait_modeA:
 	get_trace(4, 
 		"get mode A[",
-		IC7851modes_[A.imode], 
+		IC7851modes_[A.imode].c_str(), 
 		"] ", 
 		str2hex(replystr.c_str(), replystr.length()));
 
@@ -559,7 +578,7 @@ void RIG_IC7851::set_modeB(int val)
 
 	set_trace(4, 
 		"set mode B[",
-		IC7851modes_[B.imode], 
+		IC7851modes_[B.imode].c_str(), 
 		"] ", 
 		str2hex(cmd.c_str(), cmd.length()));
 }
@@ -611,7 +630,7 @@ int RIG_IC7851::get_modeB()
 end_wait_modeB:
 	get_trace(4, 
 		"get mode B[",
-		IC7851modes_[B.imode], 
+		IC7851modes_[B.imode].c_str(), 
 		"] ", 
 		str2hex(replystr.c_str(), replystr.length()));
 
@@ -705,25 +724,24 @@ int RIG_IC7851::adjust_bandwidth(int m)
 	return bw;
 }
 
-const char ** RIG_IC7851::bwtable(int m)
+std::vector<std::string>& RIG_IC7851::bwtable(int m)
 {
-	const char ** table;
 	switch (m) {
 		case 2: // AM
-			table = IC7851_am_bws;
+			return IC7851_am_bws;
 			break;
 		case 5: // FM
-			table = IC7851_fm_bws;
+			return IC7851_fm_bws;
 			break;
 		case 3: case 7: // CW
 		case 4: case 8: // RTTY
 		case 0: case 1: // SSB
 		case 12: case 13: // PKT
 		default:
-			table = IC7851_ssb_bws;
+			return IC7851_ssb_bws;
 			break;
 	}
-	return table;
+	return IC7851_ssb_bws;
 }
 
 int RIG_IC7851::def_bandwidth(int m)
