@@ -43,10 +43,12 @@ enum {
 	CW9700, CW9700R, RTTY9700, RTTY9700R,
 	LSB9700D, USB9700D, AM9700D, FM9700D, DVR9700 };
 
-const char *IC9700modes_[] = {
+static std::vector<std::string>IC9700modes_;
+static const char *vIC9700modes_[] =
+{
 "LSB", "USB", "AM", "FM", "DV", 
 "CW", "CW-R", "RTTY", "RTTY-R",
-"LSB-D", "USB-D", "AM-D", "FM-D", "DV-R", NULL};
+"LSB-D", "USB-D", "AM-D", "FM-D", "DV-R"};
 
 const char IC9700_mode_nbr[] = {
 	0x00, 0x01, 0x02, 0x05, 0x17,
@@ -58,13 +60,14 @@ const char IC9700_mode_type[] = {
 	'L', 'U', 'L', 'U',
 	'L', 'U', 'U', 'U' };
 
-const char *IC9700_ssb_bws[] = {
+static std::vector<std::string>IC9700_ssb_bws;
+static const char *vIC9700_ssb_bws[] =
+{
 "50",    "100",  "150",  "200",  "250",  "300",  "350",  "400",  "450",  "500",
 "600",   "700",  "800",  "900", "1000", "1100", "1200", "1300", "1400", "1500",
 "1600", "1700", "1800", "1900", "2000", "2100", "2200", "2300", "2400", "2500",
 "2600", "2700", "2800", "2900", "3000", "3100", "3200", "3300", "3400", "3500",
-"3600",
-NULL };
+"3600"};
 static int IC9700_bw_vals_SSB[] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
 10,11,12,13,14,15,16,17,18,19,
@@ -72,13 +75,14 @@ static int IC9700_bw_vals_SSB[] = {
 30,31,32,33,34,35,36,37,38,39,
 40, WVALS_LIMIT};
 
-const char *IC9700_am_bws[] = {
+static std::vector<std::string>IC9700_am_bws;
+static const char *vIC9700_am_bws[] =
+{
 "200",   "400",  "600",  "800", "1000", "1200", "1400", "1600", "1800", "2000",
 "2200", "2400", "2600", "2800", "3000", "3200", "3400", "3600", "3800", "4000",
 "4200", "4400", "4600", "4800", "5000", "5200", "5400", "5600", "5800", "6000",
 "6200", "6400", "6600", "6800", "7000", "7200", "7400", "9700", "7800", "8000",
-"8200", "8400", "8600", "8800", "9000", "9200", "9400", "9600", "9800", "10000", 
-NULL };
+"8200", "8400", "8600", "8800", "9000", "9200", "9400", "9600", "9800", "10000"};
 static int IC9700_bw_vals_AM[] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
 10,11,12,13,14,15,16,17,18,19,
@@ -87,19 +91,22 @@ static int IC9700_bw_vals_AM[] = {
 40,41,42,43,44,45,46,47,48,49,
 WVALS_LIMIT};
 
-const char *IC9700_rtty_bws[] = {
+static std::vector<std::string>IC9700_rtty_bws;
+static const char *vIC9700_rtty_bws[] =
+{
 "50",    "100",  "150",  "200",  "250",  "300",  "350",  "400",  "450",  "500",
 "600",   "700",  "800",  "900", "1000", "1100", "1200", "1300", "1400", "1500",
 "1600", "1700", "1800", "1900", "2000", "2100", "2200", "2300", "2400", "2500",
-"2600", "2700",
-NULL };
+"2600", "2700"};
 static int IC9700_bw_vals_RTTY[] = {
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
 10,11,12,13,14,15,16,17,18,19,
 20,21,22,23,24,25,26,27,28,29,
 30,31,  WVALS_LIMIT};
 
-const char *IC9700_fixed_bws[] = { "FIXED", NULL };
+static std::vector<std::string>IC9700_fixed_bws;
+static const char *vIC9700_fixed_bws[] =
+{ "FIXED" };
 static int IC9700_bw_vals_fixed[] = { 1, WVALS_LIMIT};
 
 static GUI IC9700_widgets[]= {
@@ -123,6 +130,19 @@ static GUI IC9700_widgets[]= {
 
 void RIG_IC9700::initialize()
 {
+	VECTOR (IC9700modes_, vIC9700modes_);
+	VECTOR (IC9700_ssb_bws, vIC9700_ssb_bws);
+	VECTOR (IC9700_am_bws, vIC9700_am_bws);
+	VECTOR (IC9700_rtty_bws, vIC9700_rtty_bws);
+	VECTOR (IC9700_fixed_bws, vIC9700_fixed_bws);
+
+	modes_ = IC9700modes_;
+
+	bandwidths_ = IC9700_ssb_bws;
+	bw_vals_ = IC9700_bw_vals_SSB;
+
+	_mode_type = IC9700_mode_type;
+
 	IC9700_widgets[0].W = btnVol;
 	IC9700_widgets[1].W = sldrVOLUME;
 	IC9700_widgets[2].W = btnAGC;
@@ -789,9 +809,9 @@ void RIG_IC9700::set_bwA(int val)
 	waitFB("set bw A");
 
 	std::stringstream ss;
-	const char *bwstr = IC9700_ssb_bws[val];
-	if ((A.imode == AM9700) || (A.imode == AM9700D)) bwstr = IC9700_am_bws[val];
-	if ((A.imode == RTTY9700) || (A.imode == RTTY9700R)) bwstr = IC9700_rtty_bws[val];
+	const char *bwstr = IC9700_ssb_bws[val].c_str();
+	if ((A.imode == AM9700) || (A.imode == AM9700D)) bwstr = IC9700_am_bws[val].c_str();
+	if ((A.imode == RTTY9700) || (A.imode == RTTY9700R)) bwstr = IC9700_rtty_bws[val].c_str();
 	 ss << "set_bwA(" << val << ") [" << bwstr << "] " <<
 		str2hex(replystr.data(), replystr.length());
 	set_trace(1, ss.str().c_str());
@@ -833,9 +853,9 @@ void RIG_IC9700::set_bwB(int val)
 	waitFB("set bw B");
 
 	std::stringstream ss;
-	const char *bwstr = IC9700_ssb_bws[val];
-	if ((B.imode == AM9700) || (B.imode == AM9700D)) bwstr = IC9700_am_bws[val];
-	if ((B.imode == RTTY9700) || (B.imode == RTTY9700R)) bwstr = IC9700_rtty_bws[val];
+	const char *bwstr = IC9700_ssb_bws[val].c_str();
+	if ((B.imode == AM9700) || (B.imode == AM9700D)) bwstr = IC9700_am_bws[val].c_str();
+	if ((B.imode == RTTY9700) || (B.imode == RTTY9700R)) bwstr = IC9700_rtty_bws[val].c_str();
 	ss << "set_bwB(" << val << ") [" << bwstr << "] " <<
 		str2hex(replystr.data(), replystr.length());
 	set_trace(1, ss.str().c_str());
@@ -877,25 +897,24 @@ int RIG_IC9700::adjust_bandwidth(int m)
 	return bw;
 }
 
-const char ** RIG_IC9700::bwtable(int m)
+std::vector<std::string>& RIG_IC9700::bwtable(int m)
 {
-	const char ** table;
 	switch (m) {
 		case AM9700: case AM9700D:
-			table = IC9700_am_bws;
+			return IC9700_am_bws;
 			break;
 		case FM9700: case FM9700D :
 		case DV9700: case DVR9700 :
-			table = IC9700_fixed_bws;
+			return IC9700_fixed_bws;
 			break;
 		case CW9700: case CW9700R:
 		case RTTY9700: case RTTY9700R:
 		case LSB9700: case USB9700:
 		case LSB9700D: case USB9700D:
 		default:
-			table = IC9700_ssb_bws;
+			return IC9700_ssb_bws;
 	}
-	return table;
+	return IC9700_ssb_bws;
 }
 
 int RIG_IC9700::def_bandwidth(int m)
