@@ -51,75 +51,91 @@ static int defBW_wide[NUM_MODES] = {
 static int mode_bwA[NUM_MODES] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
 static int mode_bwB[NUM_MODES] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1};
 
-static const char *FTdx10modes_[] = {
+static std::vector<std::string>FTdx10modes_;
+static const char *vFTdx10modes_[] = {
 "LSB", "USB", "CW-U", "FM", "AM", 
 "RTTY-L", "CW-L", "DATA-L", "RTTY-U", "DATA-FM",
-"FM-N", "DATA-U", "AM-N", "PSK", "DATA-FMN", NULL};
+"FM-N", "DATA-U", "AM-N", "PSK", "DATA-FMN"};
 
 static const char FTdx10_mode_chr[] =  { '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
 static const char FTdx10_mode_type[] = { 'L', 'U', 'U', 'U', 'U', 'L', 'L', 'L', 'U', 'U', 'U', 'U', 'U', 'U', 'U' };
 
-static const char *FTdx10_widths_SSB[] = {
-"Default",										// default width for mode
+static std::vector<std::string>FTdx10_widths_SSB;
+static const char *vFTdx10_widths_SSB[]  = {
+//"Default",										// default width for mode
  "300",  "400",  "600",  "850", "1100", 	// 1 ... 5
 "1200", "1500", "1650", "1800", "1950",		// 6 ... 10
 "2100", "2250", "2400", "2450", "2500",		// 7 ... 15
 "2600", "2700", "2800", "2900", "3000",		// 16 ... 20
-"3200", "3500", "4000", NULL };				// 21 ... 23
+"3200", "3500", "4000" };				// 21 ... 23
 
 static int FTdx10_wvals_SSB[] = {
-0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23, WVALS_LIMIT};
+//0,
+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23, WVALS_LIMIT};
 
-static const char *FTdx10_widths_CW[] = {
-"Default",										// default for mode
+static std::vector<std::string>FTdx10_widths_CW;
+static const char *vFTdx10_widths_CW[] = {
+//"Default",										// default for mode
   "50",  "100",  "150",  "200",  "250",		// 1 ... 5
  "300",  "350",  "400",  "450",  "500",		// 6 ... 10
  "600",  "800", "1200", "1400", "1700",		// 11 ... 15
-"2000", "2400", "3000", NULL };				// 16 ... 18
+"2000", "2400", "3000" };				// 16 ... 18
 
 static int FTdx10_wvals_CW[] = {
-0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
+//0,
+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
 
-static const char *FTdx10_widths_RTTY[] = {
-"Default",										// default for mode
+static std::vector<std::string>FTdx10_widths_RTTY;
+static const char *vFTdx10_widths_RTTY[] = {
+//"Default",										// default for mode
   "50",  "100",  "150",  "200",  "250",		// 1 ... 5
  "300",  "350",  "400",  "450",  "500",		// 6 ... 10
  "600",  "800", "1200", "1400", "1700",		// 11 ... 15
-"2000", "2400", "3000", NULL };				// 16 ... 18
+"2000", "2400", "3000" };				// 16 ... 18
 
 static int FTdx10_wvals_RTTY[] = {
-0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
+//0,
+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
 
-static const char *FTdx10_widths_DATA[] = {
-"Default",										// default for mode
+static std::vector<std::string>FTdx10_widths_DATA;
+static const char *vFTdx10_widths_DATA[] = {
+//"Default",										// default for mode
   "50",  "100",  "150",  "200",  "250",		// 1 ... 5
  "300",  "350",  "400",  "450",  "500",		// 6 ... 10
  "600",  "800", "1200", "1400", "1700",		// 11 ... 15
-"2000", "2400", "3000", NULL };				// 16 ... 18
+"2000", "2400", "3000" };				// 16 ... 18
 
 static int FTdx10_wvals_PSK[] = {
-0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
+//0,
+1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, WVALS_LIMIT };
 
 static const int FTdx10_wvals_AMFM[] = { 0, WVALS_LIMIT };
 
-static const char *FTdx10_widths_AMwide[] = { "9000", NULL };
-static const char *FTdx10_widths_AMnar[]  = { "6000", NULL };
-static const char *FTdx10_widths_FMnar[]  = { "9000", NULL };
-static const char *FTdx10_widths_FMwide[] = { "16000", NULL };
-static const char *FTdx10_widths_DATA_FM[]  = { "16000", NULL };
-static const char *FTdx10_widths_DATA_FMN[] = { "9000", NULL };
+static std::vector<std::string>FTdx10_widths_AMwide;
+static const char *vFTdx10_widths_AMwide[] = { "9000" };
+static std::vector<std::string>FTdx10_widths_AMnar;
+static const char *vFTdx10_widths_AMnar[]  = { "6000" };
+static std::vector<std::string>FTdx10_widths_FMnar;
+static const char *vFTdx10_widths_FMnar[]  = { "9000" };
+static std::vector<std::string>FTdx10_widths_FMwide;
+static const char *vFTdx10_widths_FMwide[] = { "16000" };
+static std::vector<std::string>FTdx10_widths_DATA_FM;
+static const char *vFTdx10_widths_DATA_FM[]  = { "16000" };
+static std::vector<std::string>FTdx10_widths_DATA_FMN;
+static const char *vFTdx10_widths_DATA_FMN[] = { "9000" };
 
-static const char *FTdx10_US_60m[] = {NULL, "126", "127", "128", "130", NULL};
+static std::vector<std::string>FTdx10_US_60m;
+static const char *vFTdx10_US_60m[] = {"", "126", "127", "128", "130"};
 // US has 5 60M presets. Using dummy numbers for all.
-// First NULL means skip 60m sets in get_band_selection().
+// First "" means skip 60m sets in get_band_selection().
 // Maybe someone can do a cat command MC; on all 5 presets and add returned numbers above.
 // To send cat commands in flrig goto menu Config->Xcvr select->Send Cmd.
 //
 // UK has 7 60M presets. Using dummy numbers for all.  If you want support,
 // Maybe someone can do a cat command MC; on all 7 presets and add returned numbers below.
-// static const char *FTdx10_UK_60m[] = {NULL, "126", "127", "128", "130", "131", "132", NULL};
+// static std::vector<std::string>FTdx10_UK_60m = {"", "126", "127", "128", "130", "131", "132"};
 
-static const char **Channels_60m = FTdx10_US_60m;
+static std::vector<std::string>& Channels_60m = FTdx10_US_60m;
 
 static GUI rig_widgets[]= {
 	{ (Fl_Widget *)btnVol,        2, 125,  50 },
@@ -138,6 +154,23 @@ static GUI rig_widgets[]= {
 
 void RIG_FTdx10::initialize()
 {
+	VECTOR (FTdx10modes_, vFTdx10modes_);
+	VECTOR (FTdx10_widths_SSB, vFTdx10_widths_SSB);
+	VECTOR (FTdx10_widths_CW, vFTdx10_widths_CW);
+	VECTOR (FTdx10_widths_RTTY, vFTdx10_widths_RTTY);
+	VECTOR (FTdx10_widths_DATA, vFTdx10_widths_DATA);
+	VECTOR (FTdx10_widths_AMwide, vFTdx10_widths_AMwide);
+	VECTOR (FTdx10_widths_AMnar, vFTdx10_widths_AMnar);
+	VECTOR (FTdx10_widths_FMnar, vFTdx10_widths_FMnar);
+	VECTOR (FTdx10_widths_FMwide, vFTdx10_widths_FMwide);
+	VECTOR (FTdx10_widths_DATA_FM, vFTdx10_widths_DATA_FM);
+	VECTOR (FTdx10_widths_DATA_FMN, vFTdx10_widths_DATA_FMN);
+	VECTOR (FTdx10_US_60m, vFTdx10_US_60m);
+
+	modes_ = FTdx10modes_;
+	bandwidths_ = FTdx10_widths_SSB;
+	bw_vals_ = FTdx10_wvals_SSB;
+
 	rig_widgets[0].W = btnVol;
 	rig_widgets[1].W = sldrVOLUME;
 	rig_widgets[2].W = sldrRFGAIN;
@@ -314,10 +347,9 @@ void RIG_FTdx10::get_band_selection(int v)
 	}
 
 	if (v == 12) {	// 5MHz 60m presets
-		if (Channels_60m[0] == NULL) return;	// no 60m Channels so skip
+		if (Channels_60m[0].empty()) return;	// no 60m Channels so skip
 		if (inc_60m) {
-			if (Channels_60m[++m_60m_indx] == NULL)
-				m_60m_indx = 0;
+			if (++m_60m_indx > (int)Channels_60m.size()) m_60m_indx = 0;
 		}
 		cmd.assign("MC").append(Channels_60m[m_60m_indx]).append(";");
 	} else {		// v == 1..11 band selection OR return to vfo mode == 0
@@ -428,7 +460,7 @@ int RIG_FTdx10::get_vfoAorB()
 	wait_char(';', 4, 100, "get vfoAorB()", ASC);
 	gett("get vfoAorB()");
 	size_t p = replystr.rfind(rsp);
-	inuse = onA;
+//	inuse = onA;
 	if (p != std::string::npos)
 		inuse = (replystr[p + 2] == '1') ? onB : onA;
 	return inuse;
@@ -833,7 +865,7 @@ int RIG_FTdx10::def_bandwidth(int m)
 	return mode_bwA[m];
 }
 
-const char ** RIG_FTdx10::bwtable(int n)
+std::vector<std::string>& RIG_FTdx10::bwtable(int n)
 {
 	switch (n) {
 		case mCW_U: case mCW_L:
