@@ -50,6 +50,8 @@
 #include "cwioUI.h"
 #include "ptt.h"
 
+#include "rigpanel.h"
+
 // The server
 using namespace XmlRpc;
 
@@ -127,23 +129,26 @@ static void freq_mode_bw()
 	int BW = 0;
 	static char szval[20];
 
-	snprintf(szval, sizeof(szval), "%d", freq);
-	tempfreq.assign("F").append((selrig->inuse == onB) ? "B:" : "A:").append(szval).append("\n");
+	try {
+		snprintf(szval, sizeof(szval), "%d", freq);
+		tempfreq.assign("F").append((selrig->inuse == onB) ? "B:" : "A:").append(szval).append("\n");
 
-	BW = (selrig->inuse == onB) ? vfoB.iBW : vfoA.iBW;
-	mode = (selrig->inuse == onB) ? vfoB.imode : vfoA.imode;
-	const char **bwt = selrig->bwtable(mode);
-	const char **dsplo = selrig->lotable(mode);
-	const char **dsphi = selrig->hitable(mode);
+		BW = (selrig->inuse == onB) ? vfoB.iBW : vfoA.iBW;
+		mode = (selrig->inuse == onB) ? vfoB.imode : vfoA.imode;
+		std::vector<std::string>& bwt = selrig->bwtable(mode);
+		std::vector<std::string>& dsplo = selrig->lotable(mode);
+		std::vector<std::string>& dsphi = selrig->hitable(mode);
 
-	tempmode.assign("M:").append(selrig->modes_ ? selrig->modes_[mode] : "none").append("\n");
+		tempmode.assign("M:").append(selrig->modes_.at(mode)).append("\n");
 
-	tempbw.assign("L:").append((BW > 256 && selrig->has_dsp_controls) ?
-						(dsplo ? dsplo[BW & 0x7F] : "") :
-						(bwt && bwt[BW]) ? bwt[BW] : "" ).append("\n");
-	tempbw.append("U:").append((BW > 256 && selrig->has_dsp_controls) ?
-						(dsphi ? dsphi[(BW >> 8) & 0x7F] : "") :
-						"").append("\n");
+		tempbw.assign("L:").append((BW > 256) ?
+							dsplo.at(BW & 0x7F) :
+							bwt.at(BW & 0x7F)).append("\n");
+		tempbw.append("U:").append((BW > 256) ?
+							dsphi.at((BW >> 8) & 0x7F) : "n/a").append("\n");
+	} catch (const std::exception& e) {
+		LOG_ERROR("%s", e.what());
+	}
 }
 
 static std::string snotch()
@@ -342,6 +347,7 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_vfo");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_vfo");
 
 		if (selrig->ICOMmainsub) {
 			if (progStatus.split && PTT) freq = vfoB.freq;
@@ -439,6 +445,8 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_AB");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_AB");
+
 		xml_trace(2, "rig_get_AB: " , ((selrig->inuse == onB) ? "B" : "A"));
 		result = (selrig->inuse == onB) ? "B" : "A";
 	}
@@ -841,7 +849,7 @@ public :
 	void execute(XmlRpcValue& params, XmlRpcValue& result) {
 		XmlRpcValue modes;
 
-		if (!xcvr_online || disable_xmlrpc->value() || !selrig->modes_) {
+		if (!xcvr_online) {
 			modes[0] = "CW";
 			modes[1] = "LSB";
 			modes[2] = "USB";
@@ -850,13 +858,22 @@ public :
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_modes");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_modes");
 
 		xml_trace(1, "rig_get_modes");
-
-		int i = 0;
-		for (const char** mode = selrig->modes_; *mode; mode++, i++)
-			modes[i] = *mode;
-		result = modes;
+		try {
+			int n = 0;
+			for (size_t i = 0; i < selrig->modes_.size(); i++)
+				if (!selrig->modes_.at(i).empty())
+					modes[n++] = selrig->modes_.at(i);
+			result = modes;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			modes[0] = "CW";
+			modes[1] = "LSB";
+			modes[2] = "USB";
+			return;
+		}
 
 		}
 
@@ -882,6 +899,7 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_sideband");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_sideband");
 
 		mode = vfo->imode;
 
@@ -905,22 +923,26 @@ public:
 	rig_get_mode(XmlRpcServer* s) : XmlRpcServerMethod("rig.get_mode", s) {}
 
 	void execute(XmlRpcValue& params, XmlRpcValue& result) {
-		if (!xcvr_online || disable_xmlrpc->value()) {
+		if (!xcvr_online) {
 			result = "USB";
 			return;
 		}
 
 		int mode;
 		wait();
-		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_mode");
+		try {
+			guard_lock serial_lock(&mutex_serial, "xml rig_get_mode");
 
-		mode = vfo->imode;
-
-		std::string result_string = "none";
-		if (selrig->modes_) result_string = selrig->modes_[mode];
-		xml_trace(2, "mode on ", ((selrig->inuse == onB) ? "B " : "A "), result_string.c_str());
-		result = result_string;
+			mode = vfo->imode;
 
+			std::string result_string = "none";
+			result_string = selrig->modes_.at(mode);
+			xml_trace(2, "mode on ", ((selrig->inuse == onB) ? "B " : "A "), result_string.c_str());
+			result = result_string;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result = "USB";
+		}
 	}
 
 	std::string help() { return std::string("returns current xcvr mode"); }
@@ -936,21 +958,26 @@ public:
 	rig_get_modeA(XmlRpcServer* s) : XmlRpcServerMethod("rig.get_modeA", s) {}
 
 	void execute(XmlRpcValue& params, XmlRpcValue& result) {
-		if (!xcvr_online || disable_xmlrpc->value()) {
+		if (!xcvr_online) {
 			result = "USB";
 			return;
 		}
 
 		int mode;
 		wait();
-		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_mode");
+		try {
+			guard_lock serial_lock(&mutex_serial, "xml rig_get_mode");
 
-		mode = vfoA.imode;
+			mode = vfoA.imode;
 
-		std::string result_string = "none";
-		if (selrig->modes_) result_string = selrig->modes_[mode];
-		xml_trace(2, "mode A ", result_string.c_str());
-		result = result_string;
+			std::string result_string = "none";
+			result_string = selrig->modes_.at(mode);
+			xml_trace(2, "mode A ", result_string.c_str());
+			result = result_string;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result = "USB";
+		}
 
 	}
 
@@ -967,21 +994,26 @@ public:
 	rig_get_modeB(XmlRpcServer* s) : XmlRpcServerMethod("rig.get_modeB", s) {}
 
 	void execute(XmlRpcValue& params, XmlRpcValue& result) {
-		if (!xcvr_online || disable_xmlrpc->value()) {
+		if (!xcvr_online) {
 			result = "USB";
 			return;
 		}
 
 		int mode;
 		wait();
-		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_mode");
+		try {
+			guard_lock serial_lock(&mutex_serial, "xml rig_get_mode");
 
-		mode = vfoB.imode;
+			mode = vfoB.imode;
 
-		std::string result_string = "none";
-		if (selrig->modes_) result_string = selrig->modes_[mode];
-		xml_trace(2, "mode B ", result_string.c_str());
-		result = result_string;
+			std::string result_string = "none";
+			result_string = selrig->modes_.at(mode);
+			xml_trace(2, "mode B ", result_string.c_str());
+			result = result_string;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result = "USB";
+		}
 
 	}
 
@@ -999,7 +1031,7 @@ public :
 
 	void execute(XmlRpcValue& params, XmlRpcValue& result) {
 
-		if (!xcvr_online || disable_xmlrpc->value() || !selrig->bandwidths_) {
+		if (!xcvr_online) {
 			XmlRpcValue bws;
 			bws[0][0] = "Bandwidth";
 			bws[0][1] = "NONE";
@@ -1010,42 +1042,44 @@ public :
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_bws");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bws");
 
-		int mode = (selrig->inuse == onB) ? vfoB.imode : vfoA.imode;
-		const char **bwt = selrig->bwtable(mode);
-		const char **dsplo = selrig->lotable(mode);
-		const char **dsphi = selrig->hitable(mode);
+		try {
+			int mode = (selrig->inuse == onB) ? vfoB.imode : vfoA.imode;
+			std::vector<std::string>& bwt = selrig->bwtable(mode);
+			std::vector<std::string>& dsplo = selrig->lotable(mode);
+			std::vector<std::string>& dsphi = selrig->hitable(mode);
 
-// single bandwidth table
-		if (!selrig->has_dsp_controls || !dsplo || !dsphi) {
-			int i = 0;
-			bws[0][0] = "Bandwidth";
-			while (*bwt) {
-				bws[0][i+1] = *bwt;
-				bwt++;
-				i++;
+	// single bandwidth table
+			if (!selrig->has_dsp_controls) {
+				bws[0][0] = "Bandwidth";
+				int n = 1;
+				for (size_t i = 0; i < bwt.size(); i++) {
+					if (!bwt.at(i).empty())
+						bws[0][n++] = bwt.at(i);
+				}
 			}
-		}
-// double table either lo/hi or center/width
-		if (selrig->has_dsp_controls && dsplo && dsphi) {
-			int i = 0;
-			int n = 1;
-			std::string control_label = selrig->SL_label;
-			control_label.append("|").append(selrig->SL_tooltip);
-			bws[0][0] = control_label.c_str();
-			while (dsplo[i]) {
-				bws[0][n] = dsplo[i];
-				n++; i++;
-			}
-			i = 0;
-			control_label.assign(selrig->SH_label);
-			control_label.append("|").append(selrig->SH_tooltip);
-			bws[1][0] = control_label.c_str();
-			n = 1;
-			while (dsphi[i]) {
-				bws[1][n] = dsphi[i];
-				n++; i++;
+	// double table either lo/hi or center/width
+			if (selrig->has_dsp_controls) {
+				int n = 1;
+				std::string control_label = selrig->SL_label;
+				control_label.append("|").append(selrig->SL_tooltip);
+				bws[0][0] = control_label.c_str();
+				for (size_t i = 0; i < dsplo.size(); i++) {
+					if (!dsplo.at(i).empty())
+						bws[0][n++] = dsplo[i];
+				}
+				control_label.assign(selrig->SH_label);
+				control_label.append("|").append(selrig->SH_tooltip);
+				bws[1][0] = control_label.c_str();
+				n = 1;
+				for (size_t i = 0; i < dsphi.size(); i++) {
+					if (!dsphi.at(i).empty())
+						bws[1][n++] = dsphi[i];
+				}
 			}
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 		result = bws;
@@ -1072,39 +1106,36 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_bw");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bw");
 
 		int BW = (selrig->inuse == onB) ? vfoB.iBW : vfoA.iBW;
 		int mode = (selrig->inuse == onB) ? vfoB.imode : vfoA.imode;
 
-		const char **bwt = selrig->bwtable(mode);
-		const char **dsplo = selrig->lotable(mode);
-		const char **dsphi = selrig->hitable(mode);
+		try {
+			if (selrig->has_int_bandwidth_control) {
+				char reply[10];
+				snprintf(reply, sizeof(reply), "%d", BW);
+				result[0] = reply;
+				std::string s1 = result[0], s2 = result[1];
+				xml_trace( 5, "bandwidth on ", ((selrig->inuse == onB) ? "B " : "A "), s1.c_str(), " | ", s2.c_str());
+				return;
+			}
 
-		int max_bwt = 0,
-			max_lotable = 0,
-			max_hitable = 0;
-		if (bwt)   while (bwt[max_bwt] != NULL) max_bwt++;
-		if (dsplo) while (dsplo[max_lotable] != NULL) max_lotable++;
-		if (dsphi) while (dsphi[max_hitable] != NULL) max_hitable++;
+			std::vector<std::string>& bwt = selrig->bwtable(mode);
+			std::vector<std::string>& dsplo = selrig->lotable(mode);
+			std::vector<std::string>& dsphi = selrig->hitable(mode);
 
-		result[0] = result[1] = "";
-		if (BW < 256 && bwt) {
-			int SB = BW & 0x7F;
-			if (SB < 0) SB = 0;
-			if (SB >= max_bwt) SB = max_bwt-1;
-			result[0] = bwt[SB];
-		}
-		else if (dsplo && dsphi) {
-			int SL = BW & 0x7F;
-			if (SL >= max_lotable) SL = max_lotable - 1;
-			if (SL < 0) SL = 0;
-
-			int SH = (BW >> 8) & 0x7F;
-			if (SH >= max_hitable) SH = max_hitable - 1;
-			if (SH < 0) SH = 0;
-
-			if (dsplo) result[0] = dsplo[SL];
-			if (dsphi) result[1] = dsphi[SH];
+			result[0] = result[1] = "";
+			if (BW < 256) {
+				result[0] = bwt[BW & 0x7F];
+			}
+			else {
+				result[0] = dsplo.at(BW & 0x7F);
+				result[1] = dsphi.at((BW >> 8) & 0x7F);
+			}
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result[0] = result[1] = "";
 		}
 		std::string s1 = result[0], s2 = result[1];
 		xml_trace( 5, "bandwidth on ", ((selrig->inuse == onB) ? "B " : "A "), s1.c_str(), " | ", s2.c_str());
@@ -1131,39 +1162,36 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_bwA");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bwA");
 
 		int BW = vfoA.iBW;
 		int mode = vfoA.imode;
 
-		const char **bwt = selrig->bwtable(mode);
-		const char **dsplo = selrig->lotable(mode);
-		const char **dsphi = selrig->hitable(mode);
+		try {
+			if (selrig->has_int_bandwidth_control) {
+				char reply[10];
+				snprintf(reply, sizeof(reply), "%d", BW);
+				result[0] = reply;
+				std::string s1 = result[0], s2 = result[1];
+				xml_trace( 4, "bandwidth on A", s1.c_str(), " | ", s2.c_str());
+				return;
+			}
 
-		int max_bwt = 0,
-			max_lotable = 0,
-			max_hitable = 0;
-		if (bwt)   while (bwt[max_bwt] != NULL) max_bwt++;
-		if (dsplo) while (dsplo[max_lotable] != NULL) max_lotable++;
-		if (dsphi) while (dsphi[max_hitable] != NULL) max_hitable++;
+			std::vector<std::string>& bwt = selrig->bwtable(mode);
+			std::vector<std::string>& dsplo = selrig->lotable(mode);
+			std::vector<std::string>& dsphi = selrig->hitable(mode);
 
-		result[0] = result[1] = "";
-		if (BW < 256 && bwt) {
-			int SB = BW & 0x7F;
-			if (SB < 0) SB = 0;
-			if (SB >= max_bwt) SB = max_bwt-1;
-			result[0] = bwt[SB];
-		}
-		else if (dsplo && dsphi) {
-			int SL = BW & 0x7F;
-			if (SL >= max_lotable) SL = max_lotable - 1;
-			if (SL < 0) SL = 0;
-
-			int SH = (BW >> 8) & 0x7F;
-			if (SH >= max_hitable) SH = max_hitable - 1;
-			if (SH < 0) SH = 0;
-
-			if (dsplo) result[0] = dsplo[SL];
-			if (dsphi) result[1] = dsphi[SH];
+			result[0] = result[1] = "";
+			if (BW < 256) {
+				result[0] = bwt.at(BW & 0x7F);
+			}
+			else {
+				result[0] = dsplo.at(BW & 0x7F);
+				result[1] = dsphi.at((BW >> 8) & 0x7F);
+			}
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result[0] = result[1] = "";
 		}
 		std::string s1 = result[0], s2 = result[1];
 		xml_trace( 4, "bandwidth on A", s1.c_str(), " | ", s2.c_str());
@@ -1191,39 +1219,37 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_bwB");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bwB");
 
 		int BW = vfoB.iBW;
 		int mode = vfoB.imode;
-
-		const char **bwt = selrig->bwtable(mode);
-		const char **dsplo = selrig->lotable(mode);
-		const char **dsphi = selrig->hitable(mode);
-
-		int max_bwt = 0,
-			max_lotable = 0,
-			max_hitable = 0;
-		if (bwt)   while (bwt[max_bwt] != NULL) max_bwt++;
-		if (dsplo) while (dsplo[max_lotable] != NULL) max_lotable++;
-		if (dsphi) while (dsphi[max_hitable] != NULL) max_hitable++;
-
 		result[0] = result[1] = "";
-		if (BW < 256 && bwt) {
-			int SB = BW & 0x7F;
-			if (SB < 0) SB = 0;
-			if (SB >= max_bwt) SB = max_bwt-1;
-			result[0] = bwt[SB];
-		}
-		else if (dsplo && dsphi) {
-			int SL = BW & 0x7F;
-			if (SL >= max_lotable) SL = max_lotable - 1;
-			if (SL < 0) SL = 0;
 
-			int SH = (BW >> 8) & 0x7F;
-			if (SH >= max_hitable) SH = max_hitable - 1;
-			if (SH < 0) SH = 0;
+		try {
+			
+			if (selrig->has_int_bandwidth_control) {
+					char reply[10];
+					snprintf(reply, sizeof(reply), "%d", BW);
+					result[0] = reply;
+					std::string s1 = result[0], s2 = result[1];
+					xml_trace( 4, "bandwidth on B", s1.c_str(), " | ", s2.c_str());
+					return;
+				}
 
-			if (dsplo) result[0] = dsplo[SL];
-			if (dsphi) result[1] = dsphi[SH];
+			std::vector<std::string>& bwt = selrig->bwtable(mode);
+			std::vector<std::string>& dsplo = selrig->lotable(mode);
+			std::vector<std::string>& dsphi = selrig->hitable(mode);
+
+			if (BW < 256) {
+				result[0] = bwt.at(BW & 0x7F);
+			}
+			else {
+				result[0] = dsplo.at(BW & 0x7F);
+				result[1] = dsphi.at((BW >> 8) & 0x7F);
+			}
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
+			result[0] = result[1] = "";
 		}
 		std::string s1 = result[0], s2 = result[1];
 		xml_trace( 4, "bandwidth on B", s1.c_str(), " | ", s2.c_str());
@@ -1623,6 +1649,8 @@ public:
 			return;
 		}
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_swap");
+		guard_lock serial_lock(&mutex_serial, "xml rig_swap");
+
 		VFOQUEUE xcvr;
 		xcvr.change = SWAP;
 		xml_trace(1, "xmlrpc SWAP");
@@ -1646,6 +1674,8 @@ public:
 		}
 
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_set_swap");
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_swap");
+
 		VFOQUEUE xcvr;
 		xcvr.change = SWAP;
 		xml_trace(1, "xmlrpc SWAP");
@@ -1668,6 +1698,8 @@ public:
 		}
 
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_set_verify_swap");
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_verify_swap");
+
 		VFOQUEUE xcvr;
 		xcvr.change = SWAP;
 		xml_trace(1, "xmlrpc SWAP");
@@ -1695,6 +1727,8 @@ public:
 		int state = int(params[0]);
 		{
 			guard_lock lock(&mutex_srvc_reqs, "xml rig_set_verify_split");
+			guard_lock serial_lock(&mutex_serial, "xml rig_set_verify_split");
+
 			VFOQUEUE xcvr_split;
 			if (state) xcvr_split.change = sON;
 			else       xcvr_split.change = sOFF;
@@ -1723,6 +1757,8 @@ public:
 		int state = int(params[0]);
 		{
 			guard_lock lock(&mutex_srvc_reqs, "xml rig_set_verify_split");
+			guard_lock serial_lock(&mutex_serial, "xml rig_set_verify_split");
+
 			VFOQUEUE xcvr_split;
 			if (state) xcvr_split.change = sON;
 			else       xcvr_split.change = sOFF;
@@ -1774,6 +1810,8 @@ public:
 		}
 
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_set_AB");
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_AB");
+
 		XCVR_STATE vfo = vfoA;
 
 		vfo.src = SRVR;
@@ -1807,6 +1845,8 @@ public:
 		}
 
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_set_verify_AB");
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_verify_AB");
+
 		XCVR_STATE vfo = vfoA;
 
 		vfo.src = SRVR;
@@ -2306,28 +2346,25 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			result = 0;
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				if (selrig->inuse == onB) {
-					serviceB(nuvals);
-				} else {
-					serviceA(nuvals);
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					if (selrig->inuse == onB) {
+						serviceB(nuvals);
+					} else {
+						serviceA(nuvals);
+					}
+					Fl::awake(updateUI);
+					result = 1;
+					break;
 				}
-				Fl::awake(updateUI);
-				result = 1;
-				break;
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 	}
@@ -2352,29 +2389,28 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				if (selrig->inuse == onB) {
-					serviceB(nuvals);
-					result = (i == selrig->get_modeB());
-				} else {
-					serviceA(nuvals);
-					result = (i == selrig->get_modeA());
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					if (selrig->inuse == onB) {
+						serviceB(nuvals);
+						result = (i == (size_t)selrig->get_modeB());
+					} else {
+						serviceA(nuvals);
+						result = (i == (size_t)selrig->get_modeA());
+					}
+					Fl::awake(updateUI);
+					break;
 				}
-				Fl::awake(updateUI);
-				break;
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
+
 		return;
 
 	}
@@ -2404,24 +2440,21 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			result = 0;
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				serviceA(nuvals);
-				Fl::awake(updateUI);
-				result = 1;
-				break;
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					serviceB(nuvals);
+					Fl::awake(updateUI);
+					result = 1;
+					break;
+				}
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 	}
@@ -2446,23 +2479,21 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				serviceA(nuvals);
-				Fl::awake(updateUI);
-				result = (i == selrig->get_modeA());
-				break;
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					serviceB(nuvals);
+					Fl::awake(updateUI);
+					result = (nuvals.imode == selrig->get_modeA());
+					break;
+				}
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 	}
@@ -2491,24 +2522,21 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			result = 0;
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				serviceB(nuvals);
-				Fl::awake(updateUI);
-				result = 1;
-				break;
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					serviceB(nuvals);
+					Fl::awake(updateUI);
+					result = 1;
+					break;
+				}
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 	}
@@ -2532,23 +2560,21 @@ public:
 		nuvals.iBW = 255;
 
 		std::string numode = (std::string)params[0];
-		int i = 0;
 
-		if (!selrig->modes_) {
-			return;
-		}
-
-		while (selrig->modes_[i] != NULL) {
-			if (numode == selrig->modes_[i]) {
-				nuvals.imode = i;
-				nuvals.iBW = selrig->def_bandwidth(i);
-				guard_lock serial_lock(&mutex_serial);
-				serviceB(nuvals);
-				Fl::awake(updateUI);
-				result = (i == selrig->get_modeB());
-				break;
+		try {
+			for (size_t i = 0; i < selrig->modes_.size(); i++) {
+				if (numode == selrig->modes_.at(i))  {
+					nuvals.imode = i;
+					nuvals.iBW = selrig->def_bandwidth(i);
+					guard_lock serial_lock(&mutex_serial);
+					serviceB(nuvals);
+					Fl::awake(updateUI);
+					result = (nuvals.imode == selrig->get_modeB());
+					break;
+				}
 			}
-			i++;
+		} catch (const std::exception& e) {
+			LOG_ERROR("%s", e.what());
 		}
 
 	}
@@ -2572,21 +2598,35 @@ public:
 		int bw = (int)params[0];
 
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_set_bandwidth");
-
-		int i = 0;
-		while (	selrig->bandwidths_[i] &&
-			atol(selrig->bandwidths_[i]) < bw) {
-			i++;
-		}
-		if (!selrig->bandwidths_[i]) i--;
-		bw = atol(selrig->bandwidths_[i]);
-
-		std::ostringstream s;
-		s << "nearest bandwidth " << selrig->bandwidths_[i];
-		xml_trace(2,"Set to ", s.str().c_str());
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_bandwidth");
 
 		XCVR_STATE nuvals;
-		nuvals.iBW = i;
+
+		if (selrig->has_int_bandwidth_control) {
+
+			int bwmod = bw % 10;
+			bw /= 10;
+			if (bwmod >= 5) ++bw;
+			bw *= 10;
+			if (bw < 50) bw = 50;
+			if (bw > 4000) bw = 4000;
+			nuvals.iBW = bw;
+
+		} else {
+			try {
+				size_t i = 0;
+				for (i = 0; i < selrig->bandwidths_.size(); i++)
+					if (atol (selrig->bandwidths_.at(i).c_str()) > bw) break;
+				if (--i < 0) i = 0;
+				bw = atol(selrig->bandwidths_.at(i).c_str());
+				nuvals.iBW = i;
+				std::ostringstream s;
+				s << "nearest bandwidth " << selrig->bandwidths_[i];
+				xml_trace(2,"Set to ", s.str().c_str());
+			} catch (const std::exception& e) {
+				LOG_ERROR("%s", e.what());
+			}
+		}
 		if (selrig->inuse == onB) {
 			nuvals.freq = vfoB.freq;
 			nuvals.imode = vfoB.imode;
@@ -2614,22 +2654,37 @@ public:
 		int bw = int(params[0]);
 
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_set_verify_bandwidth");
-
-		int i = 0;
-		while (	selrig->bandwidths_[i] &&
-			atol(selrig->bandwidths_[i]) < bw) {
-			i++;
-		}
-		if (!selrig->bandwidths_[i]) i--;
-		bw = atol(selrig->bandwidths_[i]);
-
-		std::ostringstream s;
-		s << "nearest bandwidth " << selrig->bandwidths_[i];
-		xml_trace(2,"Set to ", s.str().c_str());
+		guard_lock serial_lock(&mutex_serial, "xml rig_set_verify_bandwidth");
 
 		XCVR_STATE nuvals;
+
+		if (selrig->has_int_bandwidth_control) {
+
+			int bwmod = bw % 10;
+			bw /= 10;
+			if (bwmod >= 5) ++bw;
+			bw *= 10;
+			if (bw < 50) bw = 50;
+			if (bw > 4000) bw = 4000;
+			nuvals.iBW = bw;
+
+		} else {
+			try {
+				size_t i = 0;
+				for (i = 0; i < selrig->bandwidths_.size(); i++)
+					if (atol (selrig->bandwidths_.at(i).c_str()) > bw) break;
+				if (--i < 0) i = 0;
+				bw = atol(selrig->bandwidths_.at(i).c_str());
+				nuvals.iBW = i;
+				std::ostringstream s;
+				s << "nearest bandwidth " << selrig->bandwidths_.at(i);
+				xml_trace(2,"Set to ", s.str().c_str());
+			} catch (const std::exception& e) {
+				LOG_ERROR("%s", e.what());
+			}
+
+		}
 		int retbw;
-		nuvals.iBW = i;
 		if (selrig->inuse == onB) {
 			nuvals.freq = vfoB.freq;
 			nuvals.imode = vfoB.imode;
@@ -2641,7 +2696,7 @@ public:
 			serviceA(nuvals);
 			retbw = selrig->get_bwA();
 		}
-		result = (retbw == i);
+		result = retbw;
 	}
 
 	std::string help() { return std::string("set_bw to nearest requested"); }
@@ -2659,8 +2714,15 @@ public:
 			return;
 		}
 		int bw = (int)params[0];
+		std::vector<std::string>& widths = selrig->bandwidths_;
+		if (bw < 0 || bw >= (int) widths.size()) {
+			result = 0;
+			return;
+		}
 
 		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_bw" );
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bw");
+
 		XCVR_STATE nuvals;
 		nuvals.iBW = bw;
 		if (selrig->inuse == onB) {
@@ -2690,7 +2752,14 @@ public:
 		std::string bwstr = params;
 
 		int bw = (int)params[0];
+		std::vector<std::string>& widths = selrig->bandwidths_;
+		if (bw < 0 || bw >= (int) widths.size()) {
+			result = 0;
+			return;
+		}
+
 		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_verify_bw" );
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_bw");
 
 		XCVR_STATE nuvals;
 		int retbw;
@@ -2724,15 +2793,15 @@ public:
 		std::string bwstr = params;
 
 		int bw = (int)params[0];
-		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_BW" );
-
-		const char **widths = selrig->bandwidths_;
-		int n = 0;
-		while (widths[n]) { n++; }
-		if (bw >= n || bw < 0) {
+		std::vector<std::string>& widths = selrig->bandwidths_;
+		if (bw < 0 || bw >= (int) widths.size()) {
 			result = 0;
 			return;
 		}
+
+		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_BW" );
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_BW");
+
 		XCVR_STATE nuvals;
 		if (selrig->inuse == onB) {
 			nuvals.freq = vfoB.freq;
@@ -2764,16 +2833,15 @@ public:
 		std::string bwstr = params;
 
 		int bw = (int)params[0];
-		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_BW" );
-
-		const char **widths = selrig->bandwidths_;
-		int n = 0;
-		while (widths[n]) { n++; }
-		if (bw >= n || bw < 0) {
+		std::vector<std::string>& widths = selrig->bandwidths_;
+		if (bw < 0 || bw >= (int) widths.size()) {
 			result = 0;
 			return;
 		}
 
+		guard_lock que_lock ( &mutex_srvc_reqs, "xml rig_set_BW" );
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_BW");
+
 		XCVR_STATE nuvals;
 		int retbw;
 		nuvals.iBW = bw;
@@ -2810,6 +2878,7 @@ public:
 		int bwch = (int)params[0];
 
 		guard_lock lock(&mutex_srvc_reqs, "xml rig_mod_bandwidth");
+		guard_lock serial_lock(&mutex_serial, "xml rig_mod_bandwidth");
 
 		if (selrig->inuse == onB) {
 			nuvals.freq  = vfoB.freq;
@@ -2821,28 +2890,28 @@ public:
 			nuvals.iBW   = vfoA.iBW;
 		}
 
-		const char **widths = selrig->bandwidths_;
-		int n = 0;
-		while (widths[n]) { n++; }
+		std::vector<std::string>& widths = selrig->bandwidths_;
+		int bw = atol(widths.at(0).c_str());
+		try {
+			bw = atol(widths.at(nuvals.iBW).c_str()) + bwch;
 
-		int bw = atol(widths[nuvals.iBW]) + bwch;
-		if (bw <= 0 || bw > atol(widths[n-1])) {
-			result = 0;
-			return;
+			size_t i = 0;
+			int bwval;
+			for (i = 0; i < widths.size(); i++) {
+				if ((bwval = atol(widths.at(i).c_str())) >= bw)
+					break;
+			}
+			if (bwval > bw && i > 0) bw = atol(widths.at(--i).c_str());
+
+			std::ostringstream s;
+			s << "nearest bandwidth " << widths[i];
+			xml_trace(2,"Set to ", s.str().c_str());
+
+			nuvals.iBW = i;
+		} catch (const std::exception& e) {
+			std::cout << e.what() << '\n';
 		}
 
-		int i = 0;
-		while (	widths[i] && atol(widths[i]) < bw) {
-			i++;
-		}
-		if (!widths[i]) i--;
-		bw = atol(widths[i]);
-
-		std::ostringstream s;
-		s << "nearest bandwidth " << widths[i];
-		xml_trace(2,"Set to ", s.str().c_str());
-
-		nuvals.iBW = i;
 		if (selrig->inuse == onB) {
 			serviceB(nuvals);
 		} else {
@@ -2873,6 +2942,7 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_");
 
 		char inner[10];
 		char outer[10];
@@ -2903,6 +2973,7 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_");
 
 		char inner[10];
 		snprintf(inner, sizeof(inner), "%d", progStatus.pbt_inner);
@@ -2930,6 +3001,7 @@ public:
 
 		wait();
 		guard_lock service_lock(&mutex_srvc_reqs, "xml rig_get_");
+		guard_lock serial_lock(&mutex_serial, "xml rig_get_");
 
 		char outer[10];
 		snprintf(outer, sizeof(outer), "%d", progStatus.pbt_outer);
@@ -3250,6 +3322,44 @@ public:
 
 } rig_shutdown(&rig_server);
 
+//------------------------------------------------------------------------------
+// Execute command button, 1..24
+// Execute command button SHIFT, 25..48
+//------------------------------------------------------------------------------
+
+static int exec_btn = 0;
+static bool exec_shift = false;
+
+void btn_user(void *)
+{
+	exec_btnUser(exec_btn, exec_shift);
+}
+
+class rig_cmd : public XmlRpcServerMethod {
+public:
+	rig_cmd (XmlRpcServer* s) : XmlRpcServerMethod("rig.cmd", s) {}
+
+	void execute(XmlRpcValue& params, XmlRpcValue& result) {
+		if (!xcvr_online || disable_xmlrpc->value()) {
+			result = 0;
+			return;
+		}
+
+		exec_btn = int(params[0]);
+		exec_shift = false;
+		if (exec_btn >= 25) {
+			exec_btn -= 24;
+			exec_shift = true;
+			
+			}
+		if (exec_btn < 0 || exec_btn > 24) return;
+		Fl::awake(btn_user, (void *)0);
+
+	}
+	std::string help() { return std::string("execute command button 1..24; (shift)25..48"); }
+
+} rig_cmd(&rig_server);
+
 
 struct MLIST {
 	std::string name; std::string signature; std::string help;
@@ -3353,8 +3463,9 @@ struct MLIST {
 	{ "rig.mod_bw",       "i:i", "modify bandwidth +- to nearest new value" },
 	{ "rig.vfoA2B",       "n:n", "set vfo B to vfo A freq/mode" },
 	{ "rig.freqA2B",      "n:n", "set freq B to freq A" },
-	{ "rig.modeA2B",      "n:n", "set mode B to mode A" }
+	{ "rig.modeA2B",      "n:n", "set mode B to mode A" },
 
+	{ "rig.cmd",          "n:i", "execute command button 1..24; 25..48(shift)"}
 };
 
 class rig_list_methods : public XmlRpcServerMethod {
