@@ -433,8 +433,10 @@ void init_port_combos()
 void cbCIVdefault()
 {
 	char hexstr[8];
-	int picked = selectRig->index();
-	rigbase *srig = rigs[picked];
+	rigbase *srig = (rigbase *)(selectRig->data());
+	xcvr_name = srig->name_;
+	LOG_INFO("picked %s", xcvr_name.c_str());
+
 	snprintf(hexstr, sizeof(hexstr), "0x%02X", srig->defaultCIV);
 	txtCIV->value(hexstr);
 	progStatus.CIV = srig->defaultCIV;
@@ -1788,9 +1790,14 @@ Fl_Color flrig_def_color(int n)
 	return (Fl_Color)flrig_cmap[n];
 }
 
+extern std::string print_all();
 
 void cb_send_command(std::string command, Fl_Output *resp)
 {
+	if (command == "PRINT") {
+		std::cout << print_all();
+		return;
+	}
 	if (command.empty()) return;
 	bool usehex = false;
 
