@@ -1126,34 +1126,70 @@ void TRACED(init_Generic_Tabs)
 	poll_nr->activate(); poll_nr->value(progStatus.poll_nr);
 	poll_compression->activate(); poll_compression->value(progStatus.poll_compression);
 
-	if (!selrig->has_bandwidth_control) { poll_bandwidth->deactivate(); poll_bandwidth->value(0); }
-	if (!selrig->has_smeter) { poll_smeter->deactivate(); poll_smeter->value(0); }
-	if (!selrig->has_power_out) { poll_pout->deactivate(); poll_pout->value(0); }
-	if (!selrig->has_swr_control) { poll_swr->deactivate(); poll_swr->value(0); }
-	if (!selrig->has_alc_control) { poll_alc->deactivate(); poll_alc->value(0); }
-	if (!selrig->has_volume_control) { poll_volume->deactivate(); poll_volume->value(0); }
-	if (!selrig->has_notch_control) { poll_notch->deactivate(); poll_notch->value(0); }
+	if (!selrig->has_bandwidth_control) { 
+		poll_bandwidth->deactivate(); 
+		poll_bandwidth->value( progStatus.poll_bandwidth = 0 ); }
+	if (!selrig->has_smeter) { 
+		poll_smeter->deactivate(); 
+		poll_smeter->value( progStatus.poll_smeter = 0 ); }
+	if (!selrig->has_power_out) { 
+		poll_pout->deactivate(); 
+		poll_pout->value( progStatus.poll_pout = 0 ); }
+	if (!selrig->has_swr_control) { 
+		poll_swr->deactivate(); 
+		poll_swr->value( progStatus.poll_swr = 0 ); }
+	if (!selrig->has_alc_control) { 
+		poll_alc->deactivate(); 
+		poll_alc->value( progStatus.poll_alc = 0 ); }
+	if (!selrig->has_volume_control) { 
+		poll_volume->deactivate(); 
+		poll_volume->value( progStatus.poll_volume = 0 ); }
+	if (!selrig->has_notch_control) { 
+		poll_notch->deactivate(); 
+		poll_notch->value( progStatus.poll_notch = 0 ); }
+	if (!selrig->has_tune_control) { 
+		poll_tuner->deactivate();
+		poll_tuner->value( progStatus.poll_tuner = 0 ); }
+	if (!selrig->has_cw_break_in) { 
+		poll_break_in->deactivate(); 
+		poll_break_in->value( progStatus.poll_break_in = 0 ); }
 	if (!selrig->has_auto_notch ||
-		xcvr_name == rig_FT1000MP.name_ )
-		{ poll_auto_notch->deactivate(); poll_auto_notch->value(0); }
+		xcvr_name == rig_FT1000MP.name_ ) { 
+			poll_auto_notch->deactivate(); 
+			poll_auto_notch->value( progStatus.poll_auto_notch = 0 ); }
 	if (!selrig->has_ifshift_control) {
 		poll_ifshift->deactivate();
-		poll_ifshift->value(0);
-	}
+		poll_ifshift->value( progStatus.poll_ifshift = 0 ); }
 	if (!selrig->has_pbt_controls) { 
 		poll_pbt->deactivate(); 
-		poll_pbt->value(0);
-	}
-	if (!selrig->has_power_control) { poll_power_control->deactivate(); poll_power_control->value(0); }
-	if (!selrig->has_preamp_control && !selrig->has_attenuator_control)
-		{ poll_pre_att->deactivate(); poll_pre_att->value(0); }
-	if (!selrig->has_sql_control) { poll_squelch->deactivate(); poll_squelch->value(0); }
-	if (!selrig->has_micgain_control) { poll_micgain->deactivate(); poll_micgain->value(0); }
-	if (!selrig->has_rf_control) { poll_rfgain->deactivate(); poll_rfgain->value(0); }
-	if (!selrig->has_split) { poll_split->deactivate(); poll_split->value(0); }
-	if (!selrig->has_noise_control) {poll_noise->deactivate(); poll_noise->value(0);}
-	if (!selrig->has_noise_reduction) {poll_nr->deactivate(); poll_nr->value(0);}
-	if (!selrig->has_compression) { poll_compression->deactivate(); poll_compression->value(0); }
+		poll_pbt->value( progStatus.poll_pbt = 0 ); }
+	if (!selrig->has_power_control) { 
+		poll_power_control->deactivate(); 
+		poll_power_control->value( progStatus.poll_power_control = 0 ); }
+	if (!selrig->has_preamp_control && !selrig->has_attenuator_control) {
+		poll_pre_att->deactivate(); 
+		poll_pre_att->value( progStatus.poll_pre_att = 0 ); }
+	if (!selrig->has_sql_control) { 
+		poll_squelch->deactivate(); 
+		poll_squelch->value( progStatus.poll_squelch = 0 ); }
+	if (!selrig->has_micgain_control) { 
+		poll_micgain->deactivate(); 
+		poll_micgain->value( progStatus.poll_micgain = 0 ); }
+	if (!selrig->has_rf_control) { 
+		poll_rfgain->deactivate(); 
+		poll_rfgain->value( progStatus.poll_rfgain = 0 ); }
+	if (!selrig->has_split) { 
+		poll_split->deactivate(); 
+		poll_split->value( progStatus.poll_split = 0 ); }
+	if (!selrig->has_noise_control) {
+		poll_noise->deactivate(); 
+		poll_noise->value( progStatus.poll_noise = 0 );}
+	if (!selrig->has_noise_reduction) {
+		poll_nr->deactivate(); 
+		poll_nr->value( progStatus.poll_nr = 0 );}
+	if (!selrig->has_compression) { 
+		poll_compression->deactivate(); 
+		poll_compression->value( progStatus.poll_compression = 0 ); }
 
 }
 
@@ -1258,17 +1294,32 @@ static void cb_pbt(Fl_Counter *, void *)
 
 void TRACED(init_dsp_controls)
 
+	if (selrig->has_int_bandwidth_control) {
+		opBW->hide();
+		opDSP_lo->hide();
+		opDSP_hi->hide();
+		btnDSP->hide();
+		btnFILT->hide();
+		opBW_A->show();
+		opBW_B->show();
+		return;
+	}
+
 	if (selrig->has_dsp_controls) {
 		opDSP_lo->clear();
 		opDSP_hi->clear();
 		btnDSP->label(selrig->SL_label);
 		btnDSP->redraw_label();
-		for (int i = 0; selrig->dsp_SL[i] != NULL; i++)
-			opDSP_lo->add(selrig->dsp_SL[i]);
-		opDSP_lo->tooltip(selrig->SL_tooltip);
-		for (int i = 0; selrig->dsp_SH[i] != NULL; i++)
-			opDSP_hi->add(selrig->dsp_SH[i]);
-		opDSP_hi->tooltip(selrig->SH_tooltip);
+		try {
+			for (size_t i = 0; i < selrig->dsp_SL.size(); i++)
+				opDSP_lo->add(selrig->dsp_SL.at(i).c_str());
+			opDSP_lo->tooltip(selrig->SL_tooltip);
+			for (size_t i = 0; i < selrig->dsp_SH.size(); i++)
+				opDSP_hi->add(selrig->dsp_SH.at(i).c_str());
+			opDSP_hi->tooltip(selrig->SH_tooltip);
+		} catch (const std::exception& e) {
+			std::cout << e.what() << '\n';
+		}
 		if (vfo->iBW > 256) {
 			opDSP_lo->index(vfo->iBW & 0xFF);
 			opDSP_hi->index((vfo->iBW >> 8) & 0x7F);
@@ -1278,7 +1329,7 @@ void TRACED(init_dsp_controls)
 			opBW->hide();
 			opBW->index(0);
 			btnFILT->hide();
-		} else {
+		} else { //if ( (xcvr_name != rig_KX3.name_ )){
 			opDSP_lo->index(0);
 			opDSP_hi->index(0);
 			btnDSP->hide();
@@ -1287,7 +1338,7 @@ void TRACED(init_dsp_controls)
 			btnFILT->hide();
 			opBW->show();
 		}
-	} else if (selrig->has_FILTER) {
+	} else if (selrig->has_FILTER) { // && (xcvr_name != rig_KX3.name_) ) {
 		btnDSP->hide();
 		opDSP_lo->hide();
 		opDSP_hi->hide();
@@ -1295,7 +1346,7 @@ void TRACED(init_dsp_controls)
 		opBW->resize(opDSP_lo->x(), opDSP_lo->y(), opDSP_lo->w(), opDSP_lo->h());
 		opBW->redraw();
 		opBW->show();
-	} else {
+	} else { //if ( (xcvr_name != rig_KX3.name_) ){
 		btnDSP->hide();
 		opDSP_lo->hide();
 		opDSP_hi->hide();
@@ -1306,7 +1357,7 @@ void TRACED(init_dsp_controls)
 			selrig->name_ == rig_FLEX1500.name_ ) {
 			opCENTER->clear();
 			for (int i = 0; i < tci_nbr_centers; i++)
-				opCENTER->add(TCI_centers[i]);
+				opCENTER->add(TCI_centers[i].c_str());
 			opCENTER->show();
 			opCENTER->index(progStatus.tci_center);
 			if (progStatus.UIsize == small_ui) {
@@ -2233,7 +2284,9 @@ static void cb_bw_B(Fl_Counter *, void *)
 }
 
 void TRACED(init_K3_KX3_special)
-	if (!(xcvr_name == rig_K3.name_ || xcvr_name == rig_KX3.name_)) return;
+	if (!(xcvr_name == rig_K3.name_ || 
+		  xcvr_name == rig_KX3.name_ ||
+		  xcvr_name == rig_K4.name_ ) ) return;
 
 	if (xcvr_name == rig_K3.name_) {
 		btnB->hide();
@@ -2262,9 +2315,9 @@ void TRACED(init_K3_KX3_special)
 	}
 
 	if (progStatus.UIsize == small_ui) {
-		opBW_A->resize(214, 103, 100, 18);
+		opBW_A->resize(212, 103, 104, 18);
 		opBW_A->redraw();
-		opBW_B->resize(320, 103, 100, 18);
+		opBW_B->resize(318, 103, 104, 18);
 		opBW_B->redraw();
 	}
 	opBW_A->callback((Fl_Callback*)cb_bw_A);
@@ -2340,6 +2393,9 @@ void TRACED(initRig)
 		}
 	}
 
+trace(1, "selrig->initialize()");
+		selrig->initialize();
+
 // disable the serial thread
 	{
 		guard_lock gl_serial(&mutex_serial);
@@ -2379,12 +2435,10 @@ void TRACED(initRig)
 			progress->redraw_label();
 			update_progress(0);
 		}
-trace(1, "selrig->initialize()");
-		selrig->initialize();
 
 		if (selrig->name_ == rig_QCXP.name_) read_menus();
 
-if (!testmode) {
+if (!testmode && xcvr_name != rig_null.name_) {
 	trace(1, "selrig->check()");
 		if (!selrig->check()) {
 			trace(1, "FAILED");
@@ -2516,10 +2570,9 @@ void TRACED(init_title)
 }
 
 void TRACED(initConfigDialog)
-
-	int picked = selectRig->index();
-	rigbase *srig = rigs[picked];
+	rigbase *srig = (rigbase *)(selectRig->data());
 	xcvr_name = srig->name_;
+	LOG_INFO("picked %s", xcvr_name.c_str());
 
 	if (!progStatus.loadXcvrState(xcvr_name) ) {
 		selectCommPort->index(0);
@@ -2667,10 +2720,12 @@ void TRACED(initRigCombo)
 	selectRig->clear();
 	int i = 0;
 	while (rigs[i] != NULL) {
-		selectRig->add(rigs[i]->name_.c_str());
+		selectRig->add(rigs[i]->name_.c_str(), (void*)rigs[i]);
+//std::cout << i+1 << " : " << rigs[i]->name_ << std::endl;
 		i++;
 	}
 
-	selectRig->index(0);
+	selectRig->sort();
+	selectRig->index(selectRig->find_index(szNORIG));
 }
 
